/*
    peripherals.c
    Copyright (C) 2014
    Authors: Arun I, Hari Prasad Gokul R 

    This file is part of Quiz Buzzer v2 firmware.

    Quiz Buzzer v2 firmware is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Quiz Buzzer v2 firmware is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Quiz Buzzer v2 firmware.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <simavr/sim_io.h>
#include <simavr/avr_ioport.h>

#include "button.h"
#include "sevensegment.h"
#include "ui.h"
#include "peripherals.h"

static button_t btns[9];
static sevensegment_t display;

static void teambuttons_init(avr_t* avr);
static void display_init(avr_t* avr);

void peripherals_init(avr_t* avr)
{
	teambuttons_init(avr);
	display_init(avr);
}

void teambutton_toggle(int team)
{
	button_toggle(&btns[team]);
}

uint8_t get_displaystate()
{
	return display.common ^ display.value;
}

static void teambuttons_init(avr_t* avr)
{
	int i;
	char name[32] = "reset";
	button_init(avr, &btns[0], 1, name);
	avr_connect_irq(btns[0].irq, avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('C'), 5));
	for (i=1; i<9; i++) {
		sprintf(name, "team%d", i);
		button_init(avr, &btns[i], 1, name);
		avr_connect_irq(btns[i].irq, avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('D'), i - 1));
	}
}

static void display_init(avr_t* avr)
{
	const char name[] = "display";
	sevensegment_init(avr, &display, SEVENSEGMENT_COMMON_CATHODE, name);
	display.value = display.common;
	avr_connect_irq(avr_io_getirq(avr, AVR_IOCTL_IOPORT_GETIRQ('B'), IOPORT_IRQ_PIN_ALL), display.irq);
}
