/*
    ui.c
    Copyright (C) 2014
    Authors: Arun I, Hari Prasad Gokul R 

    This file is part of Quiz Buzzer v2 firmware.

    Quiz Buzzer v2 firmware is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Quiz Buzzer v2 firmware is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Quiz Buzzer v2 firmware.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <GL/glut.h>
#include "peripherals.h"
#include "ui.h"

#define BACKGROUND_COLOR 0.19608,0.80392,0.19608,1
#define SEGMENT_OUTER_WIDTH 0.8
#define SEGMENT_INNER_WIDTH 0.6
#define SEGMENT_HEIGHT 0.2
#define SPACING 0.05
#define DISPLAY_HEIGHT ((2*SEGMENT_OUTER_WIDTH)+SPACING)
#define DISPLAY_WIDTH (SEGMENT_OUTER_WIDTH+SPACING)

#define TICKINTERVAL 100
#define ESC_KEY 27
#define _BV(x) (1<<(x))

int state = 0;

static void render(void);
static void reshape(GLsizei width, GLsizei height);
static void keyup(unsigned char key, int x, int y);
static void keydown(unsigned char key, int x, int y);
static void tick(int value);
static void drawsegment(unsigned char seg);

void ui_init(int* argc, char** argv)
{
	glutInit(argc, argv);
	glutCreateWindow("Buzzer v2 - Simulation");
	glutDisplayFunc(render);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keydown);
	glutKeyboardUpFunc(keyup);
	glutSetKeyRepeat(GLUT_KEY_REPEAT_OFF);
	glutTimerFunc(TICKINTERVAL, tick, 0);
}

void ui_start()
{
	glutMainLoop();
}

static void render()
{
	glClearColor(BACKGROUND_COLOR);
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);

	// Draw the three horizontal segments
	glLoadIdentity();
	drawsegment('G');
	glTranslatef(0, DISPLAY_HEIGHT/2, 0);
	drawsegment('A');
	glTranslatef(0, -DISPLAY_HEIGHT, 0);
	drawsegment('D');

	// Draw the four vertical segments
	glLoadIdentity();
	glRotatef(90, 0, 0, 1);
	glTranslatef(DISPLAY_HEIGHT/4, DISPLAY_WIDTH/2, 0);
	drawsegment('F');
	glTranslatef(0, -DISPLAY_WIDTH, 0);
	drawsegment('B');
	glTranslatef(-DISPLAY_HEIGHT/2, 0, 0);
	drawsegment('C');
	glTranslatef(0, DISPLAY_WIDTH, 0);
	drawsegment('E');

	glFlush();
}

static void reshape(GLsizei width, GLsizei height)
{
	GLfloat aspect = (GLfloat)width / (GLfloat)height;
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (width >= height)
		gluOrtho2D(-aspect, aspect, -1, 1);
	else
		gluOrtho2D(-1, 1, -1/aspect, 1/aspect);
}

static void keydown(unsigned char key, int x, int y)
{
	if ((key >= '0') && (key <= '8')) {
		teambutton_toggle(key - '0');
		return;
	}
	switch (key) {
		case ESC_KEY:
		case 'q':
			exit(EXIT_SUCCESS);
			break;
		default:
			break;
	}
}

static void keyup(unsigned char key, int x, int y)
{
	if ((key >= '0') && (key <= '8')) {
		teambutton_toggle(key - '0');
		return;
	}
}

static void tick(int value)
{
	state = get_displaystate();
	glutPostRedisplay();
	glutTimerFunc(TICKINTERVAL, tick, 0);
}

static void drawsegment(unsigned char seg)
{
	if (state & _BV(seg - 'A'))
		glColor3ub(0, 0, 0);
	else glColor3ub(255, 255, 0);
	glBegin(GL_POLYGON);
	glVertex2f(-SEGMENT_OUTER_WIDTH/2, 0);
	glVertex2f(-SEGMENT_INNER_WIDTH/2, SEGMENT_HEIGHT/2);
	glVertex2f(SEGMENT_INNER_WIDTH/2, SEGMENT_HEIGHT/2);
	glVertex2f(SEGMENT_OUTER_WIDTH/2, 0);
	glVertex2f(SEGMENT_INNER_WIDTH/2, -SEGMENT_HEIGHT/2);
	glVertex2f(-SEGMENT_INNER_WIDTH/2, -SEGMENT_HEIGHT/2);
	glEnd();
}
